//
//  DataController.h
//  MobileDaxhotel
//
//  Created by Raphael Schweikert on 29.06.08.
//  Copyright 2008 sabberworm. All rights reserved.
//

#import <UIKit/UIKit.h>
#import <sqlite3.h>

@class MobileDaxhotelAppDelegate;

typedef enum {
    DAXHOTEL_DB_STATUS_EMPTY,
    DAXHOTEL_DB_STATUS_NO_CONTENT,
    DAXHOTEL_DB_STATUS_LOADED
} DAXHOTEL_DB_STATUS;

@interface DataController : NSObject {
    MobileDaxhotelAppDelegate *delegate;
    sqlite3 *db;
    
    #pragma mark Prepared Statements
    sqlite3_stmt *stmt_get_setting;
    sqlite3_stmt *stmt_get_series_id_by_identifier;
    sqlite3_stmt *stmt_get_empty_episode_ids;
    sqlite3_stmt *stmt_get_all_series;
    sqlite3_stmt *stmt_get_all_series_no_images;
    sqlite3_stmt *stmt_get_episode;
    sqlite3_stmt *stmt_get_episode_by_search;
    sqlite3_stmt *stmt_get_episode_by_series_id;
    sqlite3_stmt *stmt_get_episode_by_series_id_and_season_number;
    sqlite3_stmt *stmt_get_episode_by_episode_id;
    sqlite3_stmt *stmt_get_episode_image;
    sqlite3_stmt *stmt_get_all_episode_images;
    sqlite3_stmt *stmt_insert_series;
    sqlite3_stmt *stmt_insert_episode_skeleton;
    sqlite3_stmt *stmt_insert_episode_image;
    sqlite3_stmt *stmt_insert_setting;
    sqlite3_stmt *stmt_update_setting;
    sqlite3_stmt *stmt_update_episode_rest;
    sqlite3_stmt *stmt_count_settings;
    sqlite3_stmt *stmt_count_empty_episodes;
    sqlite3_stmt *stmt_count_series;
    sqlite3_stmt *stmt_count_seasons;
    sqlite3_stmt *stmt_count_episodes;
    sqlite3_stmt *stmt_count_episodes_by_search;
    sqlite3_stmt *stmt_count_episodes_by_series_id;
    sqlite3_stmt *stmt_count_episodes_by_series_id_and_season_number;
    sqlite3_stmt *stmt_count_episode_images;
    sqlite3_stmt *stmt_begin_transaction;
    sqlite3_stmt *stmt_end_transaction;
}

- (DataController *)initWithAppDelegate:(MobileDaxhotelAppDelegate *)delegate;
- (void)openDb;
- (void)updateDatabase;
- (void)initStatements;
- (void)beginTransaction;
- (void)endTransaction;
- (void)setStatus:(DAXHOTEL_DB_STATUS)statusCode;
- (void)setText:(NSString *)value forProperty:(NSString *)property;
- (void)setInt:(NSInteger)value forProperty:(NSString *)property;
- (NSString *)textForProperty:(NSString *)property;
- (NSInteger)intForProperty:(NSString *)property;
- (void)insertSeries:(NSString *)shortName name:(NSString *)name image:(NSData *)image;
- (void)insertEpisodeSkeleton:(NSURL *)url series:(NSUInteger)seriesId seasonNumber:(NSInteger)seasonNumber;
- (void)insertEpisodeImage:(NSInteger)episodeId image:(NSData *)image;
- (void)insertEpisodeRest:(NSInteger)episodeId discNumber:(NSInteger)discNumber productionNumber:(NSInteger)productionNumber title:(NSString *)title shortDescription:(NSString *)shortDescription description:(NSString *)description missionDate:(NSString *)missionDate airDate:(NSString *)airDate overviewImage:(NSData *)overviewImageData;
- (NSUInteger)countEmptyEpisodes;
- (NSUInteger)countSeries;
- (NSUInteger)countSeasonsForSeries:(NSUInteger) seriesId;
- (NSUInteger)countEpisodesForSeries:(NSUInteger) seriesId andSeason: (NSUInteger)seasonNumber;
- (NSUInteger)countEpisodesBySearchString:(NSString *)searchString;
- (DAXHOTEL_DB_STATUS)status;
- (NSUInteger)seriesIdByIdentifier:(NSString *)seriesIdentifier;
- (NSMutableDictionary *)emptyEpisodeUrls;
- (NSArray *)allSeriesWithImages:(BOOL)loadImages;
- (NSDictionary *)fillEpisodeFromStatement:(sqlite3_stmt *)statement;
- (NSDictionary *)episode:(NSUInteger)episodeNumber forSeries:(NSUInteger)seriesId andSeason:(NSUInteger)seasonNumber;
- (NSDictionary *)episode:(NSUInteger)episodeId;
- (NSArray *)episodesBySearchString:(NSString *)searchString;
- (NSDictionary *)randomEpisodeForSeries:(NSUInteger)seriesId;
- (NSUInteger)countImagesForEpisode:(NSUInteger)episodeId;
- (UIImage *)image:(NSUInteger)imageNumber forEpisode:(NSUInteger)episodeId;
- (NSArray *)imagesForEpisode:(NSUInteger)episodeId withStart:(NSUInteger)start andLimit:(NSUInteger)limit;
@end
