//
//  DataController.m
//  MobileDaxhotel
//
//  Created by Raphael Schweikert on 29.06.08.
//  Copyright 2008 sabberworm. All rights reserved.
//

#import "DataController.h"


@implementation DataController
- (DataController *)initWithAppDelegate:(MobileDaxhotelAppDelegate *)del {
    self = [super init];
    delegate = [del retain];
    [self openDb];
    return self;
}

- (void)openDb {
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSString *documentsDirectory = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    NSString *writableDBPath = [documentsDirectory stringByAppendingPathComponent:@"daxhotel.sqlite3"];
    BOOL forceLoad = [[NSUserDefaults standardUserDefaults] boolForKey:@"download_on_launch"] != NO;
    BOOL fileNeedsToBeCreated = ![fileManager fileExistsAtPath:writableDBPath];
    if(fileNeedsToBeCreated || forceLoad) {
        NSError *error;
        if(forceLoad && !fileNeedsToBeCreated) {
            [fileManager removeItemAtPath:writableDBPath error:&error];
        }
        if(forceLoad) {
            [[NSUserDefaults standardUserDefaults] setBool:NO forKey:@"download_on_launch"];
        }
        NSString *prototype = [[NSBundle mainBundle] pathForResource:@"mobile_daxhotel_empty" ofType:@"sqlite3"];
        [fileManager copyItemAtPath:prototype toPath:writableDBPath error:&error];
    }
    sqlite3_open([writableDBPath UTF8String], &db);
    [self initStatements];
    [self beginTransaction];
    [self updateDatabase];
    [self endTransaction];
}

- (void)updateDatabase {
    sqlite3_stmt *statement;
    
    //Add application properties
    sqlite3_prepare_v2(db, "SELECT count(*) FROM sqlite_master WHERE name='application_settings' AND type='table';", -1, &statement, NULL);
    sqlite3_step(statement);
    if(sqlite3_column_int(statement, 0) == 0) {
        sqlite3_stmt *get_status;
        sqlite3_prepare_v2(db, "SELECT status FROM application_status WHERE id = 0;", -1, &get_status, NULL);
        sqlite3_step(get_status);
        NSString *appStatus = [NSString stringWithUTF8String:(char *)sqlite3_column_text(get_status, 0)];
        sqlite3_finalize(get_status);
        
        sqlite3_stmt *delete_status;
        sqlite3_prepare_v2(db, "DROP TABLE application_status;", -1, &delete_status, NULL);
        sqlite3_step(delete_status);
        sqlite3_finalize(delete_status);
        
        sqlite3_stmt *create_settings;
        sqlite3_prepare_v2(db, "CREATE TABLE application_settings (name varchar NOT NULL PRIMARY KEY UNIQUE, value NOT NULL);", -1, &create_settings, NULL);
        sqlite3_step(create_settings);
        sqlite3_finalize(create_settings);
        
        //Preparing previously undefined statements (will cause some leaks but it does not matter in the grand scheme)
        [self initStatements];
        
        [self setInt:1 forProperty:@"version"];
        [self setText:appStatus forProperty:@"status"];
    }
    sqlite3_finalize(statement);
    
    //Update old database version
    NSInteger version = [self intForProperty:@"version"];
    NSInteger newVersion = version;
    if(version == 1) {
        //Move the show_images_button property to the database
        BOOL showImagesButton = [[NSUserDefaults standardUserDefaults] boolForKey:@"no_large_images"] == NO;
        [self setInt:showImagesButton forProperty:@"show_images_button"];
        
        //Add episode_id index to episode_images for quicker loading
        sqlite3_prepare_v2(db, "CREATE INDEX IF NOT EXISTS image_episode_id ON episode_images (episode_id);", -1, &statement, NULL);
        sqlite3_step(statement);
        sqlite3_finalize(statement);
        
        newVersion = 2;
    }
    //Update the titles to be without colons
    if(version == 2) {
        sqlite3_stmt *update_titles;
        sqlite3_prepare_v2(db, "UPDATE series SET name = replace(name, ':', '') WHERE 1;", -1, &update_titles, NULL);
        NSLog(@"Step: %i", sqlite3_step(update_titles));
        sqlite3_finalize(update_titles);
        newVersion = 3;
    }
    if(version != newVersion) {
        [self setInt:newVersion forProperty:@"version"];
    }
}

- (void)initStatements {
    sqlite3_prepare_v2(db, "SELECT value FROM application_settings WHERE name = ?;", -1, &stmt_get_setting, NULL);
    sqlite3_prepare_v2(db, "SELECT id FROM series WHERE short_name = ?;", -1, &stmt_get_series_id_by_identifier, NULL);
    sqlite3_prepare_v2(db, "SELECT id, url FROM episodes WHERE title IS NULL;", -1, &stmt_get_empty_episode_ids, NULL);
    sqlite3_prepare_v2(db, "SELECT id, short_name, name, image FROM series;", -1, &stmt_get_all_series, NULL);
    sqlite3_prepare_v2(db, "SELECT id, short_name, name FROM series;", -1, &stmt_get_all_series_no_images, NULL);
    sqlite3_prepare_v2(db, "SELECT id, series_id, season_number, disc_number, production_number, title, short_description, description, mission_date, url, air_date, overview_image FROM episodes WHERE series_id = ? AND season_number = ? ORDER BY id LIMIT 1 OFFSET ?;", -1, &stmt_get_episode_by_series_id_and_season_number, NULL);
    sqlite3_prepare_v2(db, "SELECT id, series_id, season_number, disc_number, production_number, title, short_description, description, mission_date, url, air_date, overview_image FROM episodes WHERE series_id = ? ORDER BY id LIMIT 1 OFFSET ?;", -1, &stmt_get_episode_by_series_id, NULL);
    sqlite3_prepare_v2(db, "SELECT id, series_id, season_number, disc_number, production_number, title, short_description, description, mission_date, url, air_date, overview_image FROM episodes ORDER BY id LIMIT 1 OFFSET ?;", -1, &stmt_get_episode, NULL);
    sqlite3_prepare_v2(db, "SELECT *, (length(title)-length(replace(lower(title), ?1, '')))/length(?1)*100 + (length(short_description)-length(replace(lower(short_description), ?1, '')))/length(?1)*10 + (length(description)-length(replace(lower(description), ?1, '')))/length(?1) AS relevance FROM episodes WHERE description LIKE ?2 OR title LIKE ?2 OR short_description LIKE ?2 ORDER BY relevance DESC;", -1, &stmt_get_episode_by_search, NULL);
    sqlite3_prepare_v2(db, "SELECT id, series_id, season_number, disc_number, production_number, title, short_description, description, mission_date, url, air_date, overview_image FROM episodes WHERE id = ?;", -1, &stmt_get_episode_by_episode_id, NULL);
    sqlite3_prepare_v2(db, "SELECT image FROM episode_images WHERE episode_id = ? ORDER BY id LIMIT 1 OFFSET ?;", -1, &stmt_get_episode_image, NULL);
    sqlite3_prepare_v2(db, "SELECT image FROM episode_images WHERE episode_id = ? ORDER BY id LIMIT ? OFFSET ?;", -1, &stmt_get_all_episode_images, NULL);
    sqlite3_prepare_v2(db, "INSERT INTO series (short_name, name, image) VALUES (?, ?, ?);", -1, &stmt_insert_series, NULL);
    sqlite3_prepare_v2(db, "INSERT INTO episodes (series_id, season_number, url) VALUES (?, ?, ?);", -1, &stmt_insert_episode_skeleton, NULL);
    sqlite3_prepare_v2(db, "INSERT INTO episode_images (episode_id, image) VALUES (?, ?);", -1, &stmt_insert_episode_image, NULL);
    sqlite3_prepare_v2(db, "UPDATE application_settings SET value = ? WHERE name = ?;", -1, &stmt_update_setting, NULL);
    sqlite3_prepare_v2(db, "INSERT INTO application_settings (value, name) VALUES (?, ?);", -1, &stmt_insert_setting, NULL);
    sqlite3_prepare_v2(db, "UPDATE episodes SET disc_number = ?, production_number = ?, title = ?, short_description = ?, description = ?, mission_date = ? , air_date = ?, overview_image = ? WHERE id = ?;", -1, &stmt_update_episode_rest, NULL);
    sqlite3_prepare_v2(db, "SELECT count(*) FROM application_settings WHERE name = ?;", -1, &stmt_count_settings, NULL);
    sqlite3_prepare_v2(db, "SELECT count(*) FROM episodes WHERE title IS NULL;", -1, &stmt_count_empty_episodes, NULL);
    sqlite3_prepare_v2(db, "SELECT count(*) FROM series;", -1, &stmt_count_series, NULL);
    sqlite3_prepare_v2(db, "SELECT count(DISTINCT season_number) FROM episodes WHERE series_id = ?;", -1, &stmt_count_seasons, NULL);
    sqlite3_prepare_v2(db, "SELECT count(*) FROM episodes WHERE series_id = ? AND season_number = ?;", -1, &stmt_count_episodes_by_series_id_and_season_number, NULL);
    sqlite3_prepare_v2(db, "SELECT count(*) FROM episodes WHERE series_id = ?;", -1, &stmt_count_episodes_by_series_id, NULL);
    sqlite3_prepare_v2(db, "SELECT count(*) FROM episodes;", -1, &stmt_count_episodes, NULL);
    sqlite3_prepare_v2(db, "SELECT count(*) FROM episodes WHERE description LIKE ?1 OR title LIKE ?1 OR short_description LIKE ?1;", -1, &stmt_count_episodes_by_search, NULL);
    sqlite3_prepare_v2(db, "SELECT count(*) FROM episode_images WHERE episode_id = ?;", -1, &stmt_count_episode_images, NULL);
    sqlite3_prepare_v2(db, "BEGIN TRANSACTION;", -1, &stmt_begin_transaction, NULL);
    sqlite3_prepare_v2(db, "COMMIT;", -1, &stmt_end_transaction, NULL);
}

- (void)dealloc {
    [delegate release];
    sqlite3_finalize(stmt_get_setting);
    sqlite3_finalize(stmt_get_series_id_by_identifier);
    sqlite3_finalize(stmt_get_empty_episode_ids);
    sqlite3_finalize(stmt_get_all_series);
    sqlite3_finalize(stmt_get_all_series_no_images);
    sqlite3_finalize(stmt_get_episode_by_series_id_and_season_number);
    sqlite3_finalize(stmt_get_episode_by_series_id);
    sqlite3_finalize(stmt_get_episode);
    sqlite3_finalize(stmt_get_episode_by_search);
    sqlite3_finalize(stmt_get_episode_by_episode_id);
    sqlite3_finalize(stmt_get_episode_image);
    sqlite3_finalize(stmt_get_all_episode_images);
    sqlite3_finalize(stmt_insert_series);
    sqlite3_finalize(stmt_insert_episode_skeleton);
    sqlite3_finalize(stmt_insert_episode_image);
    sqlite3_finalize(stmt_update_setting);
    sqlite3_finalize(stmt_insert_setting);
    sqlite3_finalize(stmt_update_episode_rest);
    sqlite3_finalize(stmt_count_settings);
    sqlite3_finalize(stmt_count_empty_episodes);
    sqlite3_finalize(stmt_count_series);
    sqlite3_finalize(stmt_count_seasons);
    sqlite3_finalize(stmt_count_episodes_by_series_id_and_season_number);
    sqlite3_finalize(stmt_count_episodes_by_series_id);
    sqlite3_finalize(stmt_count_episodes);
    sqlite3_finalize(stmt_count_episodes_by_search);
    sqlite3_finalize(stmt_count_episode_images);
    sqlite3_finalize(stmt_begin_transaction);
    sqlite3_finalize(stmt_end_transaction);
    sqlite3_close(db);
    [super dealloc];
}

- (void)setStatus:(DAXHOTEL_DB_STATUS)statusCode {
    NSString *status = @"E";
    if(statusCode == DAXHOTEL_DB_STATUS_LOADED) {
        status = @"L";
    } else if(statusCode == DAXHOTEL_DB_STATUS_NO_CONTENT) {
        status = @"U";
    }
    [self setText:status forProperty:@"status"];
}

- (void)setText:(NSString *)value forProperty:(NSString *)property {
    sqlite3_stmt *statement = stmt_update_setting;
    sqlite3_bind_text(stmt_count_settings, 1, [property UTF8String], -1, SQLITE_TRANSIENT);
    sqlite3_step(stmt_count_settings);
    if(sqlite3_column_int(stmt_count_settings, 0) == 0) {
        statement = stmt_insert_setting;
    }
    sqlite3_reset(stmt_count_settings);
    
    sqlite3_bind_text(statement, 1, [value UTF8String], -1, SQLITE_TRANSIENT);
    sqlite3_bind_text(statement, 2, [property UTF8String], -1, SQLITE_TRANSIENT);
    sqlite3_step(statement);
    sqlite3_reset(statement);
}

- (void)setInt:(NSInteger)value forProperty:(NSString *)property {
    sqlite3_stmt *statement = stmt_update_setting;
    sqlite3_bind_text(stmt_count_settings, 1, [property UTF8String], -1, SQLITE_TRANSIENT);
    sqlite3_step(stmt_count_settings);
    if(sqlite3_column_int(stmt_count_settings, 0) == 0) {
        statement = stmt_insert_setting;
    }
    sqlite3_reset(stmt_count_settings);
    
    sqlite3_bind_int(statement, 1, value);
    sqlite3_bind_text(statement, 2, [property UTF8String], -1, SQLITE_TRANSIENT);
    sqlite3_step(statement);
    sqlite3_reset(statement);
}

- (NSString *)textForProperty:(NSString *)property {
    sqlite3_bind_text(stmt_get_setting, 1, [property UTF8String], -1, SQLITE_TRANSIENT);
    if(sqlite3_step(stmt_get_setting) != SQLITE_ROW) {
        sqlite3_reset(stmt_get_setting);
        return nil;
    }
    NSString *result = [NSString stringWithUTF8String:(char *)sqlite3_column_text(stmt_get_setting, 0)];
    sqlite3_reset(stmt_get_setting);
    return result;
}

- (NSInteger)intForProperty:(NSString *)property {
    sqlite3_bind_text(stmt_get_setting, 1, [property UTF8String], -1, SQLITE_TRANSIENT);
    if(sqlite3_step(stmt_get_setting) != SQLITE_ROW) {
        sqlite3_reset(stmt_get_setting);
        return -1;
    }
    NSInteger result = sqlite3_column_int(stmt_get_setting, 0);
    sqlite3_reset(stmt_get_setting);
    return result;
}

- (void)beginTransaction {
    sqlite3_step(stmt_begin_transaction);
    sqlite3_reset(stmt_begin_transaction);
}

- (void)endTransaction {
    sqlite3_step(stmt_end_transaction);
    sqlite3_reset(stmt_end_transaction);
}

- (void)insertSeries:(NSString *)shortName name:(NSString *)name image:(NSData *)imageData {
    sqlite3_bind_text(stmt_insert_series, 1, [shortName UTF8String], -1, SQLITE_TRANSIENT);
    sqlite3_bind_text(stmt_insert_series, 2, [name UTF8String], -1, SQLITE_TRANSIENT);
    sqlite3_bind_blob(stmt_insert_series, 3, [imageData bytes], [imageData length], SQLITE_TRANSIENT);
    sqlite3_step(stmt_insert_series);
    sqlite3_reset(stmt_insert_series);
}

- (void)insertEpisodeSkeleton:(NSURL *)url series:(NSUInteger)seriesId seasonNumber:(NSInteger)seasonNumber {
    sqlite3_bind_int(stmt_insert_episode_skeleton, 1, seriesId);
    sqlite3_bind_int(stmt_insert_episode_skeleton, 2, seasonNumber);
    sqlite3_bind_text(stmt_insert_episode_skeleton, 3, [[url absoluteString] UTF8String], -1, SQLITE_TRANSIENT);
    sqlite3_step(stmt_insert_episode_skeleton);
    sqlite3_reset(stmt_insert_episode_skeleton);
}

- (void)insertEpisodeImage:(NSInteger)episodeId image:(NSData *)image {
    sqlite3_bind_int(stmt_insert_episode_image, 1, episodeId);
    sqlite3_bind_blob(stmt_insert_episode_image, 2, [image bytes], [image length], SQLITE_TRANSIENT);
    int reason = 0;
    if((reason = sqlite3_step(stmt_insert_episode_image)) != SQLITE_DONE) {
        @throw [NSException exceptionWithName:@"SQLite not ok" reason:[NSString stringWithFormat:@"%i: %s", reason, sqlite3_errmsg(db)] userInfo:nil];
    }
    sqlite3_reset(stmt_insert_episode_image);
}

//1: disc_number, 2: production_number, 3: title, 4: short_description, 5: description, 6: mission_date, 7: air_date, 8: overview_image
- (void)insertEpisodeRest:(NSInteger)episodeId discNumber:(NSInteger)discNumber productionNumber:(NSInteger)productionNumber title:(NSString *)title shortDescription:(NSString *)shortDescription description:(NSString *)description missionDate:(NSString *)missionDate airDate:(NSString *)airDate overviewImage:(NSData *)overviewImageData {
    sqlite3_bind_int(stmt_update_episode_rest, 1, discNumber);
    sqlite3_bind_int(stmt_update_episode_rest, 2, productionNumber);
    sqlite3_bind_text(stmt_update_episode_rest, 3, [title UTF8String], -1, SQLITE_TRANSIENT);
    sqlite3_bind_text(stmt_update_episode_rest, 4, [shortDescription UTF8String], -1, SQLITE_TRANSIENT);
    sqlite3_bind_text(stmt_update_episode_rest, 5, [description UTF8String], -1, SQLITE_TRANSIENT);
    sqlite3_bind_text(stmt_update_episode_rest, 6, [missionDate UTF8String], -1, SQLITE_TRANSIENT);
    sqlite3_bind_text(stmt_update_episode_rest, 7, [airDate UTF8String], -1, SQLITE_TRANSIENT);
    sqlite3_bind_blob(stmt_update_episode_rest, 8, [overviewImageData bytes], [overviewImageData length], SQLITE_TRANSIENT);
    sqlite3_bind_int(stmt_update_episode_rest, 9, episodeId);
    sqlite3_step(stmt_update_episode_rest);
    sqlite3_reset(stmt_update_episode_rest);
}

- (NSUInteger)countEmptyEpisodes {
    sqlite3_step(stmt_count_empty_episodes);
    NSUInteger result = sqlite3_column_int(stmt_count_empty_episodes, 0);
    sqlite3_reset(stmt_count_empty_episodes);
    return result;
}

- (NSUInteger)countSeries {
    sqlite3_step(stmt_count_series);
    NSUInteger result = sqlite3_column_int(stmt_count_series, 0);
    sqlite3_reset(stmt_count_series);
    return result;
}

- (NSUInteger)countSeasonsForSeries:(NSUInteger) seriesId {
    sqlite3_bind_int(stmt_count_seasons, 1, seriesId);
    sqlite3_step(stmt_count_seasons);
    NSUInteger result = sqlite3_column_int(stmt_count_seasons, 0);
    sqlite3_reset(stmt_count_seasons);
    return result;
}

- (NSUInteger)countEpisodesForSeries:(NSUInteger) seriesId andSeason: (NSUInteger)seasonNumber {
    sqlite3_stmt *statement = stmt_count_episodes_by_series_id_and_season_number;
    if(seasonNumber == -1) {
        statement = stmt_count_episodes_by_series_id;
        if(seriesId == -1) {
            statement = stmt_count_episodes;
        } else {
            sqlite3_bind_int(statement, 1, seriesId);
        }
    } else {
        sqlite3_bind_int(statement, 1, seriesId);
        sqlite3_bind_int(statement, 2, seasonNumber);
    }
    
    sqlite3_step(statement);
    NSUInteger result = sqlite3_column_int(statement, 0);
    sqlite3_reset(statement);
    return result;
}

- (NSUInteger)countEpisodesBySearchString:(NSString *) searchString {
    searchString = [NSString stringWithFormat:@"%%%@%%", [searchString lowercaseString]];
    sqlite3_bind_text(stmt_count_episodes_by_search, 1, [searchString UTF8String], -1, SQLITE_TRANSIENT);
    sqlite3_step(stmt_count_episodes_by_search);
    NSUInteger result = sqlite3_column_int(stmt_count_episodes_by_search, 0);
    sqlite3_reset(stmt_count_episodes_by_search);
    return result;
}

- (DAXHOTEL_DB_STATUS)status {
    DAXHOTEL_DB_STATUS result = DAXHOTEL_DB_STATUS_EMPTY;
    NSString *status = [self textForProperty:@"status"];
    NSLog(@"Status %@", status);
    if([status isEqual:@"L"]) {
        result = DAXHOTEL_DB_STATUS_LOADED;
    } else if([status isEqual:@"U"]) {
        result = DAXHOTEL_DB_STATUS_NO_CONTENT;
    }
    return result;
}

- (NSUInteger)seriesIdByIdentifier:(NSString *)seriesIdentifier {
    sqlite3_bind_text(stmt_get_series_id_by_identifier, 1, [seriesIdentifier UTF8String], -1, SQLITE_TRANSIENT);
    sqlite3_step(stmt_get_series_id_by_identifier);
    int result = sqlite3_column_int(stmt_get_series_id_by_identifier, 0);
    sqlite3_reset(stmt_get_series_id_by_identifier);
    return result;
}

- (NSMutableDictionary *)emptyEpisodeUrls {
    NSMutableDictionary *result = [[NSMutableDictionary alloc] initWithCapacity:700];
    int status = 0;
    while((status = sqlite3_step(stmt_get_empty_episode_ids)) == SQLITE_ROW) {
        [result setObject:[NSString stringWithUTF8String:(char *)sqlite3_column_text(stmt_get_empty_episode_ids, 1)] forKey:[NSNumber numberWithInt:sqlite3_column_int(stmt_get_empty_episode_ids, 0)]];
    }
    sqlite3_reset(stmt_get_empty_episode_ids);
    //NOT autoreleased!
    return result;
}

- (NSArray *)allSeriesWithImages:(BOOL)loadImages {
    NSMutableArray *result = [[NSMutableArray alloc] init];
    sqlite3_stmt *stmt = loadImages ? stmt_get_all_series : stmt_get_all_series_no_images;
    int status = 0;
    while((status = sqlite3_step(stmt)) == SQLITE_ROW) {
        NSMutableDictionary *seriesDict = [NSMutableDictionary dictionary];
        [seriesDict setObject:[NSNumber numberWithInt:sqlite3_column_int(stmt, 0)] forKey:@"id"];
        [seriesDict setObject:[NSString stringWithUTF8String:(char *)sqlite3_column_text(stmt, 1)] forKey:@"short_name"];
        [seriesDict setObject:[NSString stringWithUTF8String:(char *)sqlite3_column_text(stmt, 2)] forKey:@"name"];
        if(loadImages) {
            const void *imageData = sqlite3_column_blob(stmt, 3);
            if(imageData) {
                NSUInteger length = sqlite3_column_bytes(stmt, 3);
                [seriesDict setObject:[NSData dataWithBytes:imageData length:length] forKey:@"image"];
            } else {
                [seriesDict setObject:nil forKey:@"image"];
            }
        }
        [result addObject: seriesDict];
    }
    sqlite3_reset(stmt);
    if(status != SQLITE_DONE) {
        [result release];
        @throw [NSException exceptionWithName:@"SQLite not ok" reason:[NSString stringWithFormat:@"%i: %s", status, sqlite3_errmsg(db)] userInfo:nil];
    }
    //NOT autoreleased!
    return result;
}

- (NSDictionary *)fillEpisodeFromStatement:(sqlite3_stmt *)statement {
    NSMutableDictionary *result = [[NSMutableDictionary alloc] init];
    [result setObject:[NSNumber numberWithInt:sqlite3_column_int(statement, 0)] forKey:@"id"];
    [result setObject:[NSNumber numberWithInt:sqlite3_column_int(statement, 1)] forKey:@"series_id"];
    [result setObject:[NSNumber numberWithInt:sqlite3_column_int(statement, 2)] forKey:@"season_number"];
    [result setObject:[NSNumber numberWithInt:sqlite3_column_int(statement, 3)] forKey:@"disc_number"];
    [result setObject:[NSNumber numberWithInt:sqlite3_column_int(statement, 4)] forKey:@"production_number"];
    [result setObject:[NSString stringWithUTF8String:(char *)sqlite3_column_text(statement, 5)] forKey:@"title"];
    [result setObject:[NSString stringWithUTF8String:(char *)sqlite3_column_text(statement, 6)] forKey:@"short_description"];
    [result setObject:[NSString stringWithUTF8String:(char *)sqlite3_column_text(statement, 7)] forKey:@"description"];
    if(NULL != sqlite3_column_text(statement, 8)) {
        [result setObject:[NSString stringWithUTF8String:(char *)sqlite3_column_text(statement, 8)] forKey:@"mission_date"];
    } else {
        [result setObject:NSLocalizedString(@"unknown", @"Unknown") forKey:@"mission_date"];
    }
    [result setObject:[NSString stringWithUTF8String:(char *)sqlite3_column_text(statement, 9)] forKey:@"url"];
    if(NULL != sqlite3_column_text(statement, 10)) {
        [result setObject:[NSString stringWithUTF8String:(char *)sqlite3_column_text(statement, 10)] forKey:@"air_date"];
    } else {
        [result setObject:NSLocalizedString(@"unknown", @"Unknown") forKey:@"air_date"];
    }
    [result setObject:[NSData dataWithBytes:sqlite3_column_blob(statement, 11) length:sqlite3_column_bytes(statement, 11)] forKey:@"overview_image"];
    //NOT autoreleased!
    return result;
}

- (NSDictionary *)episode:(NSUInteger)episodeNumber forSeries:(NSUInteger)seriesId andSeason:(NSUInteger)seasonNumber {
    int bind_counter = 1;
    sqlite3_stmt *statement = stmt_get_episode_by_series_id_and_season_number;
    if(seasonNumber == -1) {
        statement = stmt_get_episode_by_series_id;
        if(seriesId == -1) {
            statement = stmt_get_episode;
        } else {
            sqlite3_bind_int(statement, bind_counter++, seriesId);
        }
    } else {
        sqlite3_bind_int(statement, bind_counter++, seriesId);
        sqlite3_bind_int(statement, bind_counter++, seasonNumber);
    }
    sqlite3_bind_int(statement, bind_counter++, episodeNumber);
    
    sqlite3_step(statement);
    NSDictionary *result = [self fillEpisodeFromStatement:statement];
    sqlite3_reset(statement);
    //Not autoreleased
    return result;
}

- (NSDictionary *)episode:(NSUInteger)episodeId {
    sqlite3_bind_int(stmt_get_episode_by_episode_id, 1, episodeId);
    
    sqlite3_step(stmt_get_episode_by_episode_id);
    NSDictionary *result = [self fillEpisodeFromStatement:stmt_get_episode_by_episode_id];
    sqlite3_reset(stmt_get_episode_by_episode_id);
    //Not autoreleased
    return result;
}

- (NSArray *)episodesBySearchString:(NSString *)searchString {
    searchString = [searchString lowercaseString];
    sqlite3_bind_text(stmt_get_episode_by_search, 1, [searchString UTF8String], -1, SQLITE_TRANSIENT);
    sqlite3_bind_text(stmt_get_episode_by_search, 2, [[NSString stringWithFormat:@"%%%@%%", searchString] UTF8String], -1, SQLITE_TRANSIENT);
    
    NSMutableArray *result = [[NSMutableArray alloc] init];
    int sqlite_status = 0;
    while((sqlite_status = sqlite3_step(stmt_get_episode_by_search)) == SQLITE_ROW) {
        [result addObject: [[self fillEpisodeFromStatement:stmt_get_episode_by_search] autorelease]];
    }
    sqlite3_reset(stmt_get_episode_by_search);
    
    //Not autoreleased
    return result;
}

- (NSDictionary *)randomEpisodeForSeries:(NSUInteger)seriesId {
    NSUInteger count = [self countEpisodesForSeries:seriesId andSeason:-1];
    NSUInteger chosenNumber = arc4random() % count;
    //NOT autoreleased!
    return [self episode:chosenNumber forSeries:seriesId andSeason:-1];
}

- (NSUInteger)countImagesForEpisode:(NSUInteger)episodeId {
    sqlite3_bind_int(stmt_count_episode_images, 1, episodeId);
    sqlite3_step(stmt_count_episode_images);
    NSUInteger result = sqlite3_column_int(stmt_count_episode_images, 0);
    sqlite3_reset(stmt_count_episode_images);
    return result;
}

- (UIImage *)image:(NSUInteger)imageNumber forEpisode:(NSUInteger)episodeId {
    sqlite3_bind_int(stmt_get_episode_image, 1, episodeId);
    sqlite3_bind_int(stmt_get_episode_image, 2, imageNumber);
    NSUInteger status = sqlite3_step(stmt_get_episode_image);
    if(status != SQLITE_ROW) {
        sqlite3_reset(stmt_get_episode_image);
        @throw [NSException exceptionWithName:@"SQLite not ok" reason:[NSString stringWithFormat:@"%i: %s", status, sqlite3_errmsg(db)] userInfo:nil];
    }
    NSData *imageData = [NSData dataWithBytes:sqlite3_column_blob(stmt_get_episode_image, 0) length:sqlite3_column_bytes(stmt_get_episode_image, 0)];
    sqlite3_reset(stmt_get_episode_image);
    return [UIImage imageWithData:imageData];
}

- (NSArray *)imagesForEpisode:(NSUInteger)episodeId withStart:(NSUInteger)start andLimit:(NSUInteger)limit {
    sqlite3_bind_int(stmt_get_all_episode_images, 1, episodeId);
    sqlite3_bind_int(stmt_get_all_episode_images, 2, limit);
    sqlite3_bind_int(stmt_get_all_episode_images, 3, start);
    NSMutableArray *result = [[NSMutableArray alloc] initWithCapacity:7]; //Default capacity: most episodes have 7 images
    int status = 0;
    while((status = sqlite3_step(stmt_get_all_episode_images)) == SQLITE_ROW) {
        NSData *imageData = [NSData dataWithBytes:sqlite3_column_blob(stmt_get_all_episode_images, 0) length:sqlite3_column_bytes(stmt_get_all_episode_images, 0)];
        [result addObject: [UIImage imageWithData:imageData]];
    }
    sqlite3_reset(stmt_get_all_episode_images);
    if(status != SQLITE_DONE) {
        [result release];
        @throw [NSException exceptionWithName:@"SQLite not ok" reason:[NSString stringWithFormat:@"%i: %s", status, sqlite3_errmsg(db)] userInfo:nil];
    }
    //NOT autoreleased!
    return result;
}

@end
