//
//  DataLoader.m
//  MobileDaxhotel
//
//  Created by Raphael Schweikert on 29.06.08.
//  Copyright 2008 sabberworm. All rights reserved.
//

#import "DataLoader.h"
#import "LoaderWindowController.h"

#define MACRO_QUOTE(x) #x
#define STRING_VALUE_BETWEEN(var, obj, key) NSString *var = [obj firstStringBetweenStart:[self fetchPropertyForKey:@MACRO_QUOTE(key ## Prefix)] andEnd:[self fetchPropertyForKey:@MACRO_QUOTE(key ## Suffix)]]
#define STRING_VALUE_BETWEEN_DECODED(var, obj, key) STRING_VALUE_BETWEEN(var, obj, key); var = [var stringWithDecodedEntities]
#define STRING_VALUE_BETWEEN_TRIMMED(var, obj, key) STRING_VALUE_BETWEEN(var, obj, key); var = [var stringByRemovingSurroundingWhitespace]
#define STRING_VALUE_BETWEEN_ESCAPED(var, obj, key) STRING_VALUE_BETWEEN(var, obj, key); var = [var stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding]
#define STRING_VALUES_BETWEEN(var, obj, key) NSArray *var = [obj allStringsBetweenStart:[self fetchPropertyForKey:@MACRO_QUOTE(key ## Prefix)] andEnd:[self fetchPropertyForKey:@MACRO_QUOTE(key ## Suffix)]]

#define FETCH_OBJECT_FROM(name, url, type, assignment) \
type *name = nil;\
do {\
    NSUInteger connect_count = 0;\
    NSURLRequest *request = [NSURLRequest requestWithURL:url];\
    NSURLResponse *response;\
    NSData *result = nil;\
    while(result == nil && (connect_count) < 2) {\
        connect_count++;\
        result = [NSURLConnection sendSynchronousRequest:request returningResponse:&response error:NULL];\
    }\
    if(result == nil) {\
        NSLog(@"Downloading url %@ (%@) into %@ failed.", url, @#url, @#name);\
        [[self loaderWindowControllerForMainThread] displayConnectionLostAlert];\
        return NO;\
    }\
    assignment;\
} while (NO)
#define FETCH_STRING_FROM(name, url) FETCH_OBJECT_FROM(name, url, NSString, name = [[NSString alloc] initWithData:result encoding:NSWindowsCP1252StringEncoding])
#define FETCH_DATA_FROM(name, url) FETCH_OBJECT_FROM(name, url, NSData, name = [result retain])

@implementation DataLoader
- (DataLoader *)initWithData:data controller:controller status:(DAXHOTEL_DB_STATUS)status {
    self = [super init];
    dataController = [data retain];
    loaderWindowController = [controller retain];
    fetchProperties = [[NSDictionary dictionaryWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"fetchProperties" ofType:@"plist"]] retain];
    baseUrl = [[NSURL alloc] initWithString:[self fetchPropertyForKey:@"baseUrl"]];
    dbStatus = status;
    return self;
}

- (void)dealloc {
    [dataController release];
    [loaderWindowController release];
    [fetchProperties release];
    [baseUrl release];
    [super dealloc];
}

- (void)main {
    if(dbStatus == DAXHOTEL_DB_STATUS_EMPTY) {
        NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
        if([self loadSkeleton] == NO) {
            return;
        }
        [dataController setStatus:DAXHOTEL_DB_STATUS_NO_CONTENT];
        [pool release];
    }
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
    [[self loaderWindowControllerForMainThread] switchToIncrementalModeMax:[[NSNumber numberWithInteger:[dataController countEmptyEpisodes]] floatValue]];
    if([self loadContents] == NO) {
        return;
    }
    [dataController setStatus:DAXHOTEL_DB_STATUS_LOADED];
    [[self loaderWindowControllerForMainThread] finishedLoading:self];
    [pool release];
}

- (BOOL)loadSkeleton {
    [self displayLoadingText:NSLocalizedString(@"dlSeries", @"DLing series descriptions") image:nil isCenter:NO synchronized:NO];
    
    //Loading basic stuff – all series identifiers
    FETCH_STRING_FROM(seriesOverviewPage, [NSURL URLWithString:[self fetchPropertyForKey:@"seriesOverviewPageUrl"]]);
    STRING_VALUES_BETWEEN(seriesIdentifiers, seriesOverviewPage, seriesOverviewIndexSearch);
    NSMutableSet *seriesIdentifiersUnique = [NSMutableSet set];
    [seriesOverviewPage release];seriesOverviewPage=nil;
    [dataController beginTransaction];
    for(NSString *seriesIdentifier in seriesIdentifiers) {
        if([seriesIdentifiersUnique containsObject:seriesIdentifier] || [@"MOV" isEqual:seriesIdentifier]) {
            continue;
        }
        [seriesIdentifiersUnique addObject:seriesIdentifier];
        NSString *url = [[NSString alloc] initWithFormat:[self fetchPropertyForKey:@"seriesIndexPageUrl"], seriesIdentifier];
        NSURL *seriesIndexPageUrl = [NSURL URLWithString:url];
        [url release];url=nil;
        FETCH_DATA_FROM(seriesImageData, ([NSURL URLWithString:[NSString stringWithFormat:[self fetchPropertyForKey:@"seriesImageUrl"], seriesIdentifier]]));
        UIImage *seriesImage = [[UIImage alloc] initWithData:seriesImageData];
        [self displayLoadingText:[NSString stringWithFormat:NSLocalizedString(@"dlSeasons", @"DLing season index"), seriesIdentifier] image:seriesImage isCenter:NO synchronized:NO];
        FETCH_STRING_FROM(seriesIndexPage, seriesIndexPageUrl);
        STRING_VALUE_BETWEEN(seriesName, seriesIndexPage, seriesIndexSeriesName);
        STRING_VALUES_BETWEEN(seasonIdentifiers, seriesIndexPage, seriesIndexSeasonNumberSearch);
        
        [dataController insertSeries:seriesIdentifier name:[seriesName stringByReplacingOccurrencesOfString:@":" withString:@""] image:seriesImageData];
        int seriesId = [dataController seriesIdByIdentifier:seriesIdentifier];
        
        [seriesIndexPage release];seriesIndexPage=nil;
        [seriesImage release];seriesImage=nil;
        [seriesImageData release];seriesImageData=nil;

        if([seasonIdentifiers count] == 0) {
            seasonIdentifiers = [NSArray arrayWithObject:@"1"];
        }
        for(NSString *seasonIdentifier in seasonIdentifiers) {
            NSInteger seasonNumber = [seasonIdentifier integerValue];
            NSURL *seasonIndexPageUrl = [NSURL URLWithString:[NSString stringWithFormat:[self fetchPropertyForKey:@"seasonIndexPageUrl"], seriesIdentifier, seasonNumber]];
            FETCH_STRING_FROM(seasonIndexPage, seasonIndexPageUrl);
            STRING_VALUES_BETWEEN(episodeUrls, seasonIndexPage, seasonIndexEpisodeUrlSearch);
            [seasonIndexPage release];seasonIndexPage=nil;
            
            for(NSString *episodeUrl in episodeUrls) {
                NSURL *episodeUrlObject = [[NSURL alloc] initWithString:episodeUrl relativeToURL:baseUrl];
                [dataController insertEpisodeSkeleton:episodeUrlObject series:seriesId seasonNumber:seasonNumber];
                [episodeUrlObject release];
            }
        }
    }
    [dataController endTransaction];
    return YES;
}

- (BOOL)loadContents {
    //NOT autoreleased
    NSMutableDictionary *unloadedEpisodeUrls = [dataController emptyEpisodeUrls];
    NSSortDescriptor *keySorter = [[NSSortDescriptor alloc] initWithKey:@"integerValue" ascending:YES];
    NSArray *allKeys = [[unloadedEpisodeUrls allKeys] sortedArrayUsingDescriptors:[NSArray arrayWithObject:keySorter]];
    [keySorter release];keySorter=nil;
    BOOL loadImages = [dataController intForProperty:@"show_images_button"];
    for(NSNumber *episodeIdObject in allKeys) {
        NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
        NSInteger episodeId = [episodeIdObject integerValue];
        NSURL *episodePageUrl = [NSURL URLWithString:[unloadedEpisodeUrls objectForKey:episodeIdObject]];
        [unloadedEpisodeUrls removeObjectForKey:episodeIdObject];
        FETCH_STRING_FROM(episodePage, episodePageUrl);
        if([(NSString *)episodePage rangeOfString:@"File Not Found - Error 404"].location != NSNotFound) {
            NSURL *fallbackPageUrl = [NSURL URLWithString:[NSString stringWithFormat:[self fetchPropertyForKey:@"fallbackPageUrl"], [episodePageUrl path]]];
            NSLog(@"Using fallback URL for %@: %@", episodePageUrl, fallbackPageUrl);
            FETCH_STRING_FROM(fallbackEpisodePage, fallbackPageUrl);
            [episodePage release];
            episodePage = fallbackEpisodePage;
        }
        
        STRING_VALUE_BETWEEN_DECODED(title, episodePage, episodeNameSearch);
        
        [dataController beginTransaction];
        NSString *loadingText = [[NSString alloc] initWithFormat:NSLocalizedString(@"dlEpisode", @"Episode loading text"), title];
        if(loadImages) {
            //Image media popups
            STRING_VALUES_BETWEEN(mediaPopupUrls, episodePage, episodeMediaPopupUrlSearch);
            NSString *popupBaseUrl = [self fetchPropertyForKey:@"episodeMediaPopupBaseUrl"];
            for(NSString *mediaPopup in mediaPopupUrls) {
                NSURL *mediaPopupUrl = [[NSURL alloc] initWithString:[popupBaseUrl stringByAppendingString:mediaPopup] relativeToURL:baseUrl];
                FETCH_STRING_FROM(mediaPopup, mediaPopupUrl);
                [mediaPopupUrl release];mediaPopupUrl = nil;
                STRING_VALUE_BETWEEN_ESCAPED(imageUrl, mediaPopup, mediaPopupImageUrlSearch);
                [mediaPopup release];mediaPopup = nil;
                if(imageUrl != nil) {
                    FETCH_DATA_FROM(image, [NSURL URLWithString:imageUrl relativeToURL:baseUrl]);
                    if(image == nil) {
                        continue;
                    }
                    [self displayLoadingText:loadingText image:[UIImage imageWithData:image] isCenter:YES synchronized:NO];
                    [dataController insertEpisodeImage:episodeId image:image];
                    [image release];
                }
            }
            [loadingText release];loadingText=nil;
        }
        
        //Small (overview image)
        STRING_VALUE_BETWEEN_ESCAPED(overviewImageUrl, episodePage, episodeImageUrlSearch);
        NSURL *overviewImageUrlObject = [NSURL URLWithString:overviewImageUrl relativeToURL:baseUrl];
        FETCH_DATA_FROM(overviewImageData, overviewImageUrlObject);
        UIImage *overviewImage = [[UIImage alloc] initWithData:overviewImageData];
        if(!loadImages) {
            [self displayLoadingText:[loadingText autorelease] image:overviewImage isCenter:YES synchronized:NO];
            loadingText=nil;
        }
        //Releases overviewImage
        UIImage *smallOverviewImage = [(DataLoader *)[self performOnMainThreadWaitUntilDone:YES] resizeOverviewImage: overviewImage];
        [overviewImage release];overviewImage = nil;
        [overviewImageData release];overviewImageData = UIImagePNGRepresentation(smallOverviewImage);
                
        [[self loaderWindowControllerForMainThread] increment];
        
        STRING_VALUE_BETWEEN_DECODED(shortDescription, episodePage, episodeTaglineSearch);
        STRING_VALUE_BETWEEN(description, episodePage, episodeDescriptionSearch);
        description = [description firstStringBetweenStart:[self fetchPropertyForKey:@"episodeInnerDescriptionDelimiter"] andEnd:[self fetchPropertyForKey:@"episodeInnerDescriptionDelimiter"]];
        //Always remove p tags, they’re almost never correct.
        description = [[[description stringByReplacingOccurrencesOfString:@"<P>" withString:@""] stringByReplacingOccurrencesOfString:@"<P align=left>" withString:@""] stringByReplacingOccurrencesOfString:@"</P>" withString:@""];
        description = [description stringByReplacingOccurrencesOfString:@"<BR>" withString:@""];
        description = [[description stringWithDecodedEntities] stringByRemovingSurroundingWhitespace];
        STRING_VALUE_BETWEEN_TRIMMED(missionDate, episodePage, episodeStarDateNumberSearch);
        if(missionDate == nil || [@"none" isEqual:missionDate] || [@"" isEqual:missionDate]) {
            missionDate = [[episodePage firstStringBetweenStart:[self fetchPropertyForKey:@"episodeMissionDateNumberSearchPrefix"] andEnd:[self fetchPropertyForKey:@"episodeMissionDateNumberSearchSuffix"]] stringByRemovingSurroundingWhitespace];
        }
        if([@"Unknown" isEqual:missionDate]) {
            missionDate = nil;
        }
        STRING_VALUE_BETWEEN_TRIMMED(airDate, episodePage, episodeOriginalAirDateNumberSearch);
        if(airDate == nil) {
            airDate = [[episodePage firstStringBetweenStart:[self fetchPropertyForKey:@"episodeAirDateNumberSearchPrefix"] andEnd:[self fetchPropertyForKey:@"episodeAirDateNumberSearchSuffix"]] stringByRemovingSurroundingWhitespace];
        }
        // for “The Cage” (http://www.startrek.com/startrek/view/series/TOS/episode/68662.html)
        if([@"" isEqual:airDate]) {
            airDate = nil;
        }
        NSInteger discNumber = [[episodePage firstStringBetweenStart:[self fetchPropertyForKey:@"episodeDVDNumberSearchPrefix"] andEnd:[self fetchPropertyForKey:@"episodeDVDNumberSearchSuffix"]] integerValue];
        NSInteger productionNumber = [[episodePage firstStringBetweenStart:[self fetchPropertyForKey:@"episodeProductionNumberSearchPrefix"] andEnd:[self fetchPropertyForKey:@"episodeProductionNumberSearchSuffix"]] integerValue];
        [dataController insertEpisodeRest:episodeId discNumber:discNumber productionNumber:productionNumber title:title shortDescription:shortDescription description:description missionDate:missionDate airDate:airDate overviewImage: overviewImageData];
        [dataController endTransaction];
        [episodePage release];
        [pool release];
    }
    [unloadedEpisodeUrls release];
    return YES;
}

- (UIImage *)resizeOverviewImage:(UIImage *) overviewImage {
    float imageFactor = 1.0/overviewImage.size.height*48;
    CGSize imageSize = CGSizeMake(overviewImage.size.width*imageFactor, overviewImage.size.height*imageFactor);
    UIImage *smallOverviewImage = [overviewImage scaleImageToSize:imageSize];
    return smallOverviewImage;
}

- (void)displayLoadingText:(NSString *)text image:(UIImage *)image isCenter:(BOOL)isCenter synchronized:(BOOL) synchronized {
    UIViewContentMode contentMode = isCenter ? UIViewContentModeCenter : UIViewContentModeScaleAspectFit;
    [[self loaderWindowControllerForMainThreadSynchronized:synchronized] displayLoadingText:text image:image contentMode:contentMode];
}

- (LoaderWindowController *) loaderWindowControllerForMainThreadSynchronized:(BOOL) synchronized {
    return (LoaderWindowController *)[loaderWindowController performOnMainThreadWaitUntilDone:synchronized];
}

- (LoaderWindowController *) loaderWindowControllerForMainThread {
    return [self loaderWindowControllerForMainThreadSynchronized:NO];
}

- (NSString *)fetchPropertyForKey:(NSString *)propertyName {
    return (NSString *)[fetchProperties objectForKey:propertyName];
}

@end
