//
//  EpisodeSelectionTableController.m
//  MobileDaxhotel
//
//  Created by Raphael Schweikert on 18.11.08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "EpisodeSelectionTableController.h"
#import "MobileDaxhotelAppDelegate.h"
#import "DetailViewController.h"

@implementation EpisodeSelectionTableController

@synthesize currentSeries;
@synthesize appDelegate;

- (void)viewDidLoad {
    [super viewDidLoad];
    self.tableView.rowHeight = 60;
    
    if(currentSeries != nil) {
        UIColor *shadeColor = [EpisodeSelectionTableController seriesColorForSeriesWithIdentifier:[currentSeries objectForKey:@"short_name"] asWeb:NO];
        [[[self navigationController] navigationBar] setTintColor:shadeColor];
        [[self.navigationController.navigationBar.subviews objectAtIndex:1] setNeedsDisplay];
    }
}


- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return [[appDelegate dataController] countSeasonsForSeries:[(NSNumber *)[currentSeries valueForKey:@"id"] integerValue]]+1;
}

- (NSArray *)sectionIndexTitlesForTableView:(UITableView *)tableView {
    int count = [[appDelegate dataController] countSeasonsForSeries:[(NSNumber *)[currentSeries valueForKey:@"id"] integerValue]];
    if(count < 2) {
        return nil;
    }
    NSMutableArray *result = [NSMutableArray arrayWithCapacity:count];
    for(int i=1;i<=count;i++) {
        [result addObject:[NSString stringWithFormat:@"%i", i]];
    }
    return result;
}

- (NSInteger)tableView:(UITableView *)tableView sectionForSectionIndexTitle:(NSString *)title atIndex:(NSInteger)index {
    return index+1;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    if(section == 0) {
        return nil;
    }
    if([[appDelegate dataController] countSeasonsForSeries:[(NSNumber *)[currentSeries valueForKey:@"id"] integerValue]] < 2) {
        return nil;
    }
    return [NSString stringWithFormat:NSLocalizedString(@"season", @"season"), section];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    if(section == 0) {
        return 1;
    }
    return [[appDelegate dataController] countEpisodesForSeries:[(NSNumber *)[currentSeries valueForKey:@"id"] integerValue] andSeason:section];
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    if([indexPath section] == 0) {
        UITableViewCell *cell = [[[UITableViewCell alloc] initWithFrame:CGRectZero] autorelease];
        cell.image = [UIImage imageWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"shuffle" ofType:@"png"]];
        cell.text = NSLocalizedString(@"random", @"random");
        return cell;
    }
    NSDictionary *episode = [self episodeForRowAtIndexPath:indexPath];
    static NSString *CellIdentifier = @"EPISODE_CELL";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:CellIdentifier] autorelease];
    }
    
    cell.text = [episode objectForKey:@"title"];
    cell.image = [UIImage imageWithData:[episode objectForKey:@"overview_image"]];
    cell.lineBreakMode = UILineBreakModeTailTruncation;
    if([self shouldColorCells])
        cell.textColor = [EpisodeSelectionTableController seriesColorForSeriesWithIdentifier:[[appDelegate.mainWindowTableController seriesWithSeriesId:[[episode valueForKey:@"series_id"] integerValue]] valueForKey:@"short_name"] asWeb:NO];
    
    [episode release];
    return cell;
}

- (NSDictionary *)episodeForRowAtIndexPath:(NSIndexPath *)indexPath {
    return [[appDelegate dataController] episode:[indexPath row] forSeries:[(NSNumber *)[currentSeries valueForKey:@"id"] integerValue] andSeason:[indexPath section]];
}

- (BOOL)shouldColorCells {
    return NO;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    NSDictionary *episode;
    if([indexPath section] == 0) {
        episode = [[appDelegate dataController] randomEpisodeForSeries:[(NSNumber *)[currentSeries valueForKey:@"id"] integerValue]];
    } else {
        episode = [[appDelegate dataController] episode:[indexPath row] forSeries:[(NSNumber *)[currentSeries valueForKey:@"id"] integerValue] andSeason:[indexPath section]];
    }
    
    DetailViewController *detailView = appDelegate.detailView;
    [[self navigationController] pushViewController:detailView animated:YES];
    [detailView displayEpisode:episode ofSeries:currentSeries updateTintColor:NO];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    self.navigationItem.title = [currentSeries valueForKey:@"name"];
}

- (NSUInteger)currentSeriesId {
    return [(NSNumber *)[currentSeries valueForKey:@"id"] integerValue];
}

+ (UIColor *)seriesColorForSeriesWithIdentifier:(NSString *)seriesIdentifier asWeb:(BOOL)asWeb {
    static NSDictionary *seriesColors;
    if(seriesColors == nil) {
        seriesColors = [[NSDictionary alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"seriesColors" ofType:@"plist"]];
    }
    
    NSData *colorData = [seriesColors objectForKey: seriesIdentifier];
    if(colorData != nil) {
        const void *color;[colorData getBytes:&color];
        int colorValue = (unsigned int)color;
        if(asWeb) {
            return [NSString stringWithFormat:@"rgb(%i, %i, %i)", colorValue&0xff, (colorValue&0xff00)>>8, (colorValue&0xff0000)>>16];
        }
        return [UIColor colorWithRed:1.0/255*(colorValue&0xff) green:1.0/255*((colorValue&0xff00)>>8) blue:1.0/255*((colorValue&0xff0000)>>16) alpha:1.0];
    }
    return nil;
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

- (void)dealloc {
    [super dealloc];
}


@end

