//
//  MainWindowTableController.m
//  MobileDaxhotel
//
//  Created by rasch on 7/10/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "MainWindowTableController.h"
#import "MobileDaxhotelAppDelegate.h"
#import "DataController.h"
#import "SearchResultViewController.h"

@implementation MainWindowTableController

@synthesize appDelegate;

- (id)initWithStyle:(UITableViewStyle)style {
    if (self = [super initWithStyle:style]) {
    }
    return self;
}

- (NSArray *)series {
    if(series == nil) {
        series = [(DataController *)[[self appDelegate] dataController] allSeriesWithImages:NO];
    }
    
    return series;
}


- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 1;
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    DataController *dataController = [[self appDelegate] dataController];
    return [dataController countSeries]+1;
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *CellIdentifier = @"SERIES_REUSE_ID";
    
    UITableViewCell *result = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if(result == nil) {
        result = [[[UITableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:CellIdentifier] autorelease];
    }
    if([indexPath row] == 0) {
        UIImage *seriesImage = [UIImage imageWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"home_shuffle" ofType:@"png"]];
        result.image = seriesImage;
    } else {
        NSArray *seriesList = [self series];
        NSDictionary *seriesEntry = [seriesList objectAtIndex:[indexPath row]-1];
        NSString *shortName = [seriesEntry objectForKey:@"short_name"];
        UIImage *seriesImage = [UIImage imageWithContentsOfFile:[[NSBundle mainBundle] pathForResource:[@"series_" stringByAppendingString:shortName] ofType:@"png"]];
        if(seriesImage != nil) {
            result.image = seriesImage;
        } else {
            result.text = [seriesEntry objectForKey:@"name"];
        }
    }
    
    //Center the image
    if(result.image != nil) {
        result.indentationWidth = ((result.frame.size.width - (result.image.size.width))/2)-20.0f;
        result.indentationLevel = 1;
    }
    result.selectionStyle = UITableViewCellSelectionStyleBlue;
    return result;
}


- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    if([indexPath row] == 0) {
        NSDictionary *episode = [[[appDelegate dataController] randomEpisodeForSeries:-1] autorelease];
        DetailViewController *detailView = appDelegate.detailView;
        NSDictionary *seriesOfEpisode = [self seriesWithSeriesId:[[episode valueForKey:@"series_id"] integerValue]];
        [self.navigationController pushViewController:detailView animated:YES];
        [detailView displayEpisode:episode ofSeries:seriesOfEpisode updateTintColor:YES];
        return;
    }
    NSArray *seriesList = [self series];
    NSDictionary *seriesEntry = [seriesList objectAtIndex:[indexPath row]-1];
    [self pushSeriesControllerForSeries: seriesEntry animated:YES];
}

- (NSDictionary *)seriesWithSeriesId:(NSUInteger)seriesId {
    NSDictionary *foundSeries = nil;
    for(foundSeries in [self series]) {
        if([[foundSeries valueForKey:@"id"] integerValue] == seriesId) {
            break;
        }
    }
    return foundSeries;
}

- (void)pushSeriesControllerForSeries:(NSDictionary *)seriesEntry animated:(BOOL)animated {
    EpisodeSelectionTableController *controller = [[EpisodeSelectionTableController alloc] init];
    controller.currentSeries = seriesEntry;
    controller.appDelegate = appDelegate;
    [[self navigationController] pushViewController:[controller autorelease] animated:animated];
}

- (IBAction)activateSearch {
    static SearchResultViewController *searchResultViewController;
    if(!searchResultViewController) {
        searchResultViewController = [[SearchResultViewController alloc] init];
        searchResultViewController.appDelegate = self.appDelegate;
    }
    [[self navigationController] pushViewController:searchResultViewController animated:YES];
}

- (void)dealloc {
    [series release];
    [super dealloc];
}


- (void)viewDidLoad {
    [super viewDidLoad];
}


- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [[[self navigationController] navigationBar] setTintColor:nil];
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated {
}

- (void)viewDidDisappear:(BOOL)animated {
}

- (void)didReceiveMemoryWarning {
    [series release];series=nil;
    [super didReceiveMemoryWarning];
}


@end

